﻿// Reference: Humanlights.System
// Requires: aMAZEingPro

using Humanlights;
using Humanlights.Extensions;
using Oxide.Game.Rust.Cui;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Oxide.Plugins
{
    [Info ( "aMAZEingProCUI", "Raul-Sorin Sorban", "1.1.0" )]
    [Description ( "aMAZEing CUI interface for the Pro version." )]
    public class aMAZEingProCUI : RustPlugin
    {
        public const string EnabledColor = "1 0.3 0 1";
        public const string DisabledColor = "0 0 0 0.975";
        public const string GreyColor = "0.3 0.3 0.3 1";
        public const string CUI = "aMAZEingProCUI";
        public const string PreviousModeCmd = CUI + "prevmode";
        public const string NextModeCmd = CUI + "nextmode";
        public const string EditModeCmd = CUI + "editmode";
        public const string PlayModeCmd = CUI + "playmode";
        public const string ModePanelCmd = CUI + "modepanel";
        public const string AdminPanelCmd = CUI + "adminpanel";
        public const string CloseCmd = CUI + "remove";

        public Dictionary<ulong, PlayerCustomization> Players { get; private set; } = new Dictionary<ulong, PlayerCustomization> ();

        public PlayerCustomization GetPlayer ( BasePlayer player )
        {
            var customization = ( PlayerCustomization )null;
            if ( !Players.ContainsKey ( player.userID ) ) Players.Add ( player.userID, customization ?? ( customization = new PlayerCustomization () ) );
            else return Players [ player.userID ];

            return customization;
        }

        public class PlayerCustomization
        {
            public int ModeIndex { get; set; }
        }

        #region Commands

        [ConsoleCommand ( PreviousModeCmd )]
        private void PreviousMode ( ConsoleSystem.Arg arg )
        {
            var player = arg.Player ();
            var customization = GetPlayer ( player );
            customization.ModeIndex--;

            if ( customization.ModeIndex < 0 ) customization.ModeIndex = aMAZEingPro.Instance.Config.Modes.Count - 1;

            CreateModeList ( player );
        }

        [ConsoleCommand ( NextModeCmd )]
        private void NextMode ( ConsoleSystem.Arg arg )
        {
            var player = arg.Player ();
            var customization = GetPlayer ( player );

            customization.ModeIndex++;

            if ( customization.ModeIndex > aMAZEingPro.Instance.Config.Modes.Count - 1 ) customization.ModeIndex = 0;

            CreateModeList ( player );
        }

        [ConsoleCommand ( PlayModeCmd )]
        private void PlayMode ( ConsoleSystem.Arg arg )
        {
            var player = arg.Player ();
            var customization = GetPlayer ( player );

            aMAZEingPro.Instance.Call ( "Play", player, string.Empty, new string [] { aMAZEingPro.Instance.Config.Modes.ToArray () [ customization.ModeIndex ].Key } );

            Clear ( player );
        }

        [ConsoleCommand ( ModePanelCmd )]
        private void ModePanel ( ConsoleSystem.Arg arg )
        {
            var player = arg.Player ();

            CreateModeList ( player );
        }

        [ConsoleCommand ( EditModeCmd )]
        private void EditModePanel ( ConsoleSystem.Arg arg )
        {
            var player = arg.Player ();
            var customization = GetPlayer ( player );

            CreateEditModePanel ( player, aMAZEingPro.Instance.Config.Modes.ToArray () [ customization.ModeIndex ].Value );
        }

        [ConsoleCommand ( AdminPanelCmd )]
        private void AdminPanel ( ConsoleSystem.Arg arg )
        {
            var player = arg.Player ();

            CreateAdminPanel ( player );
        }

        [ConsoleCommand ( CloseCmd )]
        private void Close ( ConsoleSystem.Arg arg )
        {
            Clear ( arg.Player () );
        }

        #endregion

        #region CUI

        public string CreateBase ( CuiElementContainer container, string title )
        {
            var parent = container.Add ( new CuiPanel { Image = { Color = $"0 0 0 0.25", Material = "assets/content/ui/uibackgroundblur.mat" }, RectTransform = { AnchorMin = "0.2 0.2", AnchorMax = "0.8 0.9" }, CursorEnabled = true }, "Overlay", CUI );
            var parent2 = container.Add ( new CuiPanel { Image = { Color = "0.1 0.1 0.1 0.95", }, RectTransform = { AnchorMin = "0.01 0.02", AnchorMax = "0.99 0.975" }, CursorEnabled = true }, parent );
            container.Add ( new CuiPanel { Image = { Color = "0.05 0.05 0.05 0.9", }, RectTransform = { AnchorMin = "0.01 0.1", AnchorMax = "0.98 0.8" }, CursorEnabled = true }, parent2 );

            container.Add ( new CuiLabel { Text = { Text = $"<b><color=orange>aMAZEing</color>[<color=orange>Pro</color>]</b>", FontSize = 35, Font = "robotocondensed-regular.ttf", Align = TextAnchor.UpperCenter }, RectTransform = { AnchorMin = $"0 0.3", AnchorMax = $"1 0.96" } }, parent2 );
            container.Add ( new CuiLabel { Text = { Text = title, FontSize = 20, Font = "robotocondensed-regular.ttf", Align = TextAnchor.UpperCenter }, RectTransform = { AnchorMin = $"0 0.3", AnchorMax = $"1 0.885" } }, parent2 );

            return parent2;
        }
        public void CreateFooter ( CuiElementContainer container, string parent, bool showAdminPanel = true )
        {
            container.Add ( new CuiButton { Button = { Command = CloseCmd, Color = "0.3 0.3 0.3 1" }, RectTransform = { AnchorMin = $"0.45 0.025", AnchorMax = $"0.55 0.075" }, Text = { Text = $"CLOSE", FontSize = 13, Align = TextAnchor.MiddleCenter, Color = "1 1 1 1" } }, parent );
            // container.Add ( new CuiButton { Button = { Command = showAdminPanel ? AdminPanelCmd : ModePanelCmd, Color = EnabledColor }, RectTransform = { AnchorMin = $"0.875 0.025", AnchorMax = $"0.98 0.075" }, Text = { Text = showAdminPanel ? "ADMIN PANEL" : "MODE PANEL", FontSize = 10, Align = TextAnchor.MiddleCenter, Color = "1 1 1 1" } }, parent );
        }

        private void CreateModeList ( BasePlayer player )
        {
            Clear ( player );

            var customization = GetPlayer ( player );
            var container = new CuiElementContainer ();
            var parent = CreateBase ( container, "Modes" );
            var mode = aMAZEingPro.Instance.Config.Modes.ToArray () [ customization.ModeIndex ];
            var completionBalance = "";
            var doorUnlockingBalance = "";
            var losingBalance = "";
            var leavingBalance = "";
            switch ( mode.Value.Rewards.CurrencyType )
            {
                case aMAZEingPro.ModeRewards.CurrencyTypes.Economics:
                    completionBalance = $"{mode.Value.Rewards.CompletionBalance} Coins";
                    doorUnlockingBalance = $"{mode.Value.Rewards.DoorUnlockingBalance} Coins";
                    losingBalance = $"-{mode.Value.Rewards.LosingBalance} Coins";
                    leavingBalance = $"-{mode.Value.Rewards.LeavingBalance} Coins";
                    break;
                case aMAZEingPro.ModeRewards.CurrencyTypes.ServerRewards:
                    if ( mode.Value.Rewards.CompletionBalance == 0 ) completionBalance = $"0 RP"; else completionBalance = $"{mode.Value.Rewards.CompletionBalance / 100} RP";
                    if ( mode.Value.Rewards.DoorUnlockingBalance == 0 ) doorUnlockingBalance = $"0 RP"; else doorUnlockingBalance = $"{mode.Value.Rewards.DoorUnlockingBalance / 100} RP";
                    if ( mode.Value.Rewards.LosingBalance == 0 ) losingBalance = $"0 RP"; else losingBalance = $"-{mode.Value.Rewards.LosingBalance / 100} RP";
                    if ( mode.Value.Rewards.LeavingBalance == 0 ) leavingBalance = $"0 RP"; else leavingBalance = $"-{mode.Value.Rewards.LeavingBalance / 100} RP";
                    break;
            }
            container.Add ( new CuiLabel { Text = { Text = $"<b>{mode.Value.DisplayName}</b>", FontSize = 20, Font = "robotocondensed-regular.ttf", Align = TextAnchor.UpperLeft }, RectTransform = { AnchorMin = $"0.025 0", AnchorMax = $"1 0.775" } }, parent );
            container.Add ( new CuiLabel { Text = { Text = $"{( string.IsNullOrEmpty ( mode.Value.DisplayDescription ) ? "No description." : mode.Value.DisplayDescription )}", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.UpperLeft }, RectTransform = { AnchorMin = $"0.025 0", AnchorMax = $"1 0.725" } }, parent );
            container.Add ( new CuiLabel
            {
                Text = { Text = $"<size=17>Rewards</size>\n" +
                    $"Completion: <b>{completionBalance}</b>\n" +
                    $"Door Unlocking: <b>{doorUnlockingBalance}</b>\n" +
                    $"Losing: <b>{losingBalance}</b>\n" +
                    $"Leaving: <b>{leavingBalance}</b>\n\n" +
                    $"<size=17>Lives</size>\n" +
                    $"Host Max. Lives: <b>{mode.Value.HostMaximumLives}</b>\n" +
                    $"Help Max. Lives: <b>{mode.Value.HelperMaximumLives}</b>\n", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.LowerLeft },
                RectTransform = { AnchorMin = $"0.025 0.25", AnchorMax = $"1 1" }
            }, parent );

            var min = "0.89 0.725";
            var max = "0.97 0.78";
            container.Add ( new CuiPanel { Image = { Color = !mode.Value.Disabled ? EnabledColor : DisabledColor, }, RectTransform = { AnchorMin = min, AnchorMax = max }, CursorEnabled = true }, parent );
            { container.Add ( new CuiLabel { Text = { Text = $"ENABLED", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleCenter }, RectTransform = { AnchorMin = min, AnchorMax = max } }, parent ); }

            min = "0.83 0.725";
            max = "0.88 0.78";
            container.Add ( new CuiPanel { Image = { Color = EnabledColor }, RectTransform = { AnchorMin = min, AnchorMax = max }, CursorEnabled = true }, parent );
            { container.Add ( new CuiLabel { Text = { Text = $"{mode.Value.Size}x{mode.Value.Size}", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleCenter }, RectTransform = { AnchorMin = min, AnchorMax = max } }, parent ); }

            min = "0.78 0.725";
            max = "0.82 0.78";
            container.Add ( new CuiPanel { Image = { Color = mode.Value.Floors.Length > 0 ? EnabledColor : DisabledColor }, RectTransform = { AnchorMin = min, AnchorMax = max }, CursorEnabled = true }, parent );
            { container.Add ( new CuiLabel { Text = { Text = $"↑{mode.Value.Floors.Length}↑", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleCenter }, RectTransform = { AnchorMin = min, AnchorMax = max } }, parent ); }

            min = "0.68 0.725";
            max = "0.77 0.78";
            container.Add ( new CuiPanel { Image = { Color = EnabledColor }, RectTransform = { AnchorMin = min, AnchorMax = max }, CursorEnabled = true }, parent );
            { container.Add ( new CuiLabel { Text = { Text = $"{mode.Value.Group}", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleCenter }, RectTransform = { AnchorMin = min, AnchorMax = max } }, parent ); }

            min = "0.58 0.725";
            max = "0.67 0.78";
            container.Add ( new CuiPanel { Image = { Color = mode.Value.EnableInvites ? EnabledColor : DisabledColor }, RectTransform = { AnchorMin = min, AnchorMax = max }, CursorEnabled = true }, parent );
            { container.Add ( new CuiLabel { Text = { Text = $"INVITES", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleCenter }, RectTransform = { AnchorMin = min, AnchorMax = max } }, parent ); }

            var leaderboardParent = container.Add ( new CuiPanel { Image = { Color = "0.05 0.05 0.05 0.9", }, RectTransform = { AnchorMin = "0.58 0.1225", AnchorMax = "0.97 0.7" }, CursorEnabled = true }, parent );
            CreateLeaderboard ( player, container, DistinctBy ( aMAZEingPro.Instance.Leaderboard.Games.Where ( x => x.Mode == mode.Key ).OrderBy ( x => x.CompletionSeconds ), x => x.Player.SteamId ).ToArray (), leaderboardParent );

            if ( !mode.Value.Disabled && mode.Value.IsValid () ) container.Add ( new CuiButton { Button = { Command = PlayModeCmd, Color = EnabledColor }, RectTransform = { AnchorMin = "0.025 0.2", AnchorMax = $"0.3 0.25" }, Text = { Text = $"PLAY", FontSize = 13, Align = TextAnchor.MiddleCenter, Color = "1 1 1 1" } }, parent );
            container.Add ( new CuiButton { Button = { Command = PreviousModeCmd, Color = GreyColor }, RectTransform = { AnchorMin = "0.025 0.125", AnchorMax = $"0.152 0.175" }, Text = { Text = $"⇇ {GetPreviousModeName ( player )}", FontSize = 13, Align = TextAnchor.MiddleCenter, Color = "1 1 1 1" } }, parent );
            container.Add ( new CuiButton { Button = { Command = NextModeCmd, Color = GreyColor }, RectTransform = { AnchorMin = "0.165 0.125", AnchorMax = $"0.3 0.175" }, Text = { Text = $"{GetNextModeName ( player )} ⇉", FontSize = 13, Align = TextAnchor.MiddleCenter, Color = "1 1 1 1" } }, parent );
            // container.Add ( new CuiButton { Button = { Command = EditModeCmd, Color = GreyColor }, RectTransform = { AnchorMin = "0.315 0.125", AnchorMax = $"0.38 0.175" }, Text = { Text = $"EDIT", FontSize = 10, Align = TextAnchor.MiddleCenter, Color = "1 1 1 1" } }, parent );

            CreateFooter ( container, parent );
            CuiHelper.AddUi ( player, container );
        }
        private void CreateAdminPanel ( BasePlayer player )
        {
            Clear ( player );

            var customization = GetPlayer ( player );
            var container = new CuiElementContainer ();
            var parent = CreateBase ( container, "Admin Panel" );

            CreateFooter ( container, parent, false );
            CuiHelper.AddUi ( player, container );
        }
        private void CreateEditModePanel ( BasePlayer player, aMAZEingPro.Mode mode )
        {
            Clear ( player );

            var customization = GetPlayer ( player );
            var container = new CuiElementContainer ();
            var parent = CreateBase ( container, $"Mode Editor <size=11>— {mode.DisplayName}</size>" );

            GetInput ( container, "Mode Name", mode.DisplayName, "0.02 0.7", "0.35 0.78", parent );

            CreateFooter ( container, parent, false );
            CuiHelper.AddUi ( player, container );
        }
        private void CreateLeaderboard ( BasePlayer player, CuiElementContainer container, aMAZEingPro.RootLeaderboard.RootInstance [] instances, string parent )
        {
            var odd = false;
            var subtracter = 0.05;
            var playerYMin = 0.85 - subtracter;
            var playerYMax = 0.9 - subtracter;

            var titleBackground = container.Add ( new CuiPanel { Image = { Color = "0 0 0 0", }, RectTransform = { AnchorMin = "0.02 0.9", AnchorMax = "0.98 1" }, CursorEnabled = true }, parent );
            {
                container.Add ( new CuiLabel { Text = { Text = $"  <b>Leaderboard</b>", FontSize = 18, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleLeft }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"1 1" } }, titleBackground );
            }

            var indicatorBackground = container.Add ( new CuiPanel { Image = { Color = "0 0 0 0.8" }, RectTransform = { AnchorMin = "0.02 0.85", AnchorMax = "0.98 0.9" }, CursorEnabled = true }, parent );
            {
                container.Add ( new CuiLabel { Text = { Text = $"      <b>Player Name</b>", FontSize = 12, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleLeft }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"1 1" } }, indicatorBackground );
                container.Add ( new CuiLabel { Text = { Text = $"<b>Date</b>", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleRight }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"0.675 1" } }, indicatorBackground );
                container.Add ( new CuiLabel { Text = { Text = $"<b>Time</b>", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleRight }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"0.8 1" } }, indicatorBackground );
                container.Add ( new CuiLabel { Text = { Text = $"<b>Helpers</b>", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleRight }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"0.98 1" } }, indicatorBackground );
            }

            for ( int i = 0; i < 17; i++ )
            {
                var playerBackground = container.Add ( new CuiPanel { Image = { Color = $"0 0 0 {( odd ? 0.5 : 0 )}" }, RectTransform = { AnchorMin = $"0.02 {playerYMin}", AnchorMax = $"0.98 {playerYMax}" }, CursorEnabled = true }, parent );
                {
                    if ( i <= instances.Length - 1 )
                    {
                        var instance = instances [ i ];
                        container.Add ( new CuiLabel { Text = { Text = $"  {i + 1} ", FontSize = 10, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleLeft }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"0.65 1" } }, playerBackground );
                        container.Add ( new CuiLabel { Text = { Text = $"      {instance.Player.DisplayName}", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleLeft }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"0.65 1" } }, playerBackground );
                        container.Add ( new CuiLabel { Text = { Text = $"{new DateTick ( instance.Ticks ).Date.DateFormat2}", FontSize = 10, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleRight }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"0.675 1" } }, playerBackground );
                        container.Add ( new CuiLabel { Text = { Text = $"{TimeEx.Format ( instance.CompletionSeconds )}", FontSize = 10, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleRight }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"0.8 1" } }, playerBackground );
                        container.Add ( new CuiLabel { Text = { Text = $"{( instance.HelperPlayers == null || instance.HelperPlayers.Length == 0 ? "None" : instance.HelperPlayers.Select ( x => $"{x.DisplayName}" ).ToArray ().ToString ( ", ", " and " ).Truncate ( 20, "..." ) )}", FontSize = 10, Font = "robotocondensed-regular.ttf", Align = TextAnchor.MiddleRight }, RectTransform = { AnchorMin = $"0 0", AnchorMax = $"0.98 1" } }, playerBackground );
                    }
                }

                playerYMin -= subtracter;
                playerYMax -= subtracter;

                odd = !odd;
            }
        }
        private string GetPreviousModeName ( BasePlayer player )
        {
            var customization = GetPlayer ( player );
            var index = customization.ModeIndex - 1;
            if ( index < 0 ) index = aMAZEingPro.Instance.Config.Modes.Count - 1;

            return aMAZEingPro.Instance.Config.Modes.ToArray () [ index ].Value.DisplayName;
        }
        private string GetNextModeName ( BasePlayer player )
        {
            var customization = GetPlayer ( player );
            var index = customization.ModeIndex + 1;
            if ( index > aMAZEingPro.Instance.Config.Modes.Count - 1 ) index = 0;

            return aMAZEingPro.Instance.Config.Modes.ToArray () [ index ].Value.DisplayName;
        }
        private void GetInput ( CuiElementContainer container, string title, string value, string anchorMin, string anchorMax, string parent )
        {
            var subPanel = container.Add ( new CuiPanel { Image = { Color = "0 0 0 0.8", }, RectTransform = { AnchorMin = anchorMin, AnchorMax = anchorMax }, CursorEnabled = true }, parent );
            {
                container.Add ( new CuiLabel { Text = { Text = $"{title}", FontSize = 11, Font = "robotocondensed-regular.ttf", Align = TextAnchor.UpperLeft }, RectTransform = { AnchorMin = "0.01 0", AnchorMax = "1 0.95" } }, subPanel );

                container.Add ( new CuiElement
                {
                    Name = CuiHelper.GetGuid (),
                    Parent = subPanel,
                    Components =
                    {
                        new CuiInputFieldComponent
                        {
                            Align = TextAnchor.MiddleLeft,
                            CharsLimit = 50,
                            Color = GreyColor,
                            Command = "",
                            FontSize = 13,
                            IsPassword = false,
                            Text = value
                        },
                        new CuiRectTransformComponent {AnchorMin = "0.01 0", AnchorMax = "1 0.8" }
                    }
                } );
            }
        }

        public void Clear ( BasePlayer player )
        {
            CuiHelper.DestroyUi ( player, CUI );
        }

        #endregion

        #region Extensions

        private static IEnumerable<TSource> DistinctBy<TSource, TKey> ( IEnumerable<TSource> source, Func<TSource, TKey> keySelector )
        {
            var seenKeys = new HashSet<TKey> ();
            foreach ( TSource element in source )
            {
                if ( seenKeys.Add ( keySelector ( element ) ) )
                {
                    yield return element;
                }
            }
        }

        #endregion
    }
}
